# stdlib
from typing import Any
from typing import Optional

# third party
from google.protobuf.reflection import GeneratedProtocolMessageType

# syft relative
from ... import deserialize
from ... import serialize
from ...core.common import UID
from ...core.common.serde.serializable import bind_protobuf
from ...proto.lib.python.float_pb2 import Float as Float_PB
from .primitive_factory import PrimitiveFactory
from .primitive_interface import PyPrimitive
from .types import SyPrimitiveRet


@bind_protobuf
class Float(float, PyPrimitive):
    def __new__(cls, value: Any = None, id: Optional[UID] = None) -> "Float":
        if value is None:
            value = 0.0
        return float.__new__(cls, value)

    def __init__(self, value: Any = None, id: Optional[UID] = None):
        if value is None:
            value = 0.0

        float.__init__(value)

        self._id: UID = id if id else UID()

    @property
    def id(self) -> UID:
        """We reveal PyPrimitive.id as a property to discourage users and
        developers of Syft from modifying .id attributes after an object
        has been initialized.

        :return: returns the unique id of the object
        :rtype: UID
        """
        return self._id

    def upcast(self) -> float:
        return float(self)

    def __eq__(self, other: Any) -> SyPrimitiveRet:
        result = super().__eq__(other)
        return PrimitiveFactory.generate_primitive(value=result)

    def __ge__(self, other: Any) -> SyPrimitiveRet:
        result = super().__ge__(other)
        return PrimitiveFactory.generate_primitive(value=result)

    def __lt__(self, other: Any) -> SyPrimitiveRet:
        result = super().__lt__(other)
        return PrimitiveFactory.generate_primitive(value=result)

    def __le__(self, other: Any) -> SyPrimitiveRet:
        result = super().__le__(other)
        return PrimitiveFactory.generate_primitive(value=result)

    def __gt__(self, other: Any) -> SyPrimitiveRet:
        result = super().__gt__(other)
        return PrimitiveFactory.generate_primitive(value=result)

    def __add__(self, other: Any) -> SyPrimitiveRet:
        res = super().__add__(other)
        return PrimitiveFactory.generate_primitive(value=res)

    def __abs__(self) -> SyPrimitiveRet:
        res = super().__abs__()
        return PrimitiveFactory.generate_primitive(value=res)

    def __bool__(self) -> SyPrimitiveRet:
        res = super().__bool__()
        return PrimitiveFactory.generate_primitive(value=res)

    def __radd__(self, other: Any) -> SyPrimitiveRet:
        res = super().__radd__(other)
        return PrimitiveFactory.generate_primitive(value=res)

    def __sub__(self, other: Any) -> SyPrimitiveRet:
        res = super().__sub__(other)
        return PrimitiveFactory.generate_primitive(value=res)

    def __rsub__(self, other: Any) -> SyPrimitiveRet:
        res = super().__rsub__(other)
        return PrimitiveFactory.generate_primitive(value=res)

    def __mul__(self, other: Any) -> SyPrimitiveRet:
        res = super().__mul__(other)
        return PrimitiveFactory.generate_primitive(value=res)

    def __rmul__(self, other: Any) -> SyPrimitiveRet:
        res = super().__rmul__(other)
        return PrimitiveFactory.generate_primitive(value=res)

    def __divmod__(self, other: Any) -> SyPrimitiveRet:
        value = super().__divmod__(other)
        return PrimitiveFactory.generate_primitive(value=value)

    def __neg__(self) -> SyPrimitiveRet:
        res = super().__neg__()
        return PrimitiveFactory.generate_primitive(value=res)

    def __ne__(self, other: Any) -> SyPrimitiveRet:
        res = super().__ne__(other)
        return PrimitiveFactory.generate_primitive(value=res)

    def __floordiv__(self, other: Any) -> SyPrimitiveRet:
        res = super().__floordiv__(other)
        return PrimitiveFactory.generate_primitive(value=res)

    def __truediv__(self, other: Any) -> SyPrimitiveRet:
        res = super().__truediv__(other)
        return PrimitiveFactory.generate_primitive(value=res)

    def __mod__(self, other: Any) -> SyPrimitiveRet:
        res = super().__mod__(other)
        return PrimitiveFactory.generate_primitive(value=res)

    def __rmod__(self, other: Any) -> SyPrimitiveRet:
        res = super().__rmod__(other)
        return PrimitiveFactory.generate_primitive(value=res)

    def __rdivmod__(self, other: Any) -> SyPrimitiveRet:
        res = super().__rdivmod__(other)
        return PrimitiveFactory.generate_primitive(value=res)

    def __rfloordiv__(self, other: Any) -> SyPrimitiveRet:
        res = super().__rfloordiv__(other)
        return PrimitiveFactory.generate_primitive(value=res)

    def __round__(self, n: Optional[int] = None) -> SyPrimitiveRet:
        res = super().__round__(n)
        return PrimitiveFactory.generate_primitive(value=res)

    def __rtruediv__(self, other: Any) -> SyPrimitiveRet:
        res = super().__rtruediv__(other)
        return PrimitiveFactory.generate_primitive(value=res)

    def __trunc__(self) -> SyPrimitiveRet:
        res = super().__trunc__()
        return PrimitiveFactory.generate_primitive(value=res)

    def as_integer_ratio(self) -> SyPrimitiveRet:
        tpl = super().as_integer_ratio()
        return PrimitiveFactory.generate_primitive(value=tpl)

    def is_integer(self) -> SyPrimitiveRet:
        res = super().is_integer()
        return PrimitiveFactory.generate_primitive(value=res)

    def __pow__(self, other: Any, modulo: Optional[Any] = None) -> SyPrimitiveRet:
        if modulo:
            return PrimitiveFactory.generate_primitive(
                value=super().__pow__(other, modulo)
            )
        return PrimitiveFactory.generate_primitive(value=super().__pow__(other))

    def __rpow__(self, other: Any, modulo: Optional[Any] = None) -> SyPrimitiveRet:
        if modulo:
            return PrimitiveFactory.generate_primitive(
                value=super().__rpow__(other, modulo)
            )
        return PrimitiveFactory.generate_primitive(value=super().__rpow__(other))

    def __iadd__(self, other: Any) -> SyPrimitiveRet:
        return PrimitiveFactory.generate_primitive(
            value=super().__add__(other), id=self.id
        )

    def __isub__(self, other: Any) -> SyPrimitiveRet:
        return PrimitiveFactory.generate_primitive(
            value=super().__sub__(other), id=self.id
        )

    def __imul__(self, other: Any) -> SyPrimitiveRet:
        return PrimitiveFactory.generate_primitive(
            value=super().__mul__(other), id=self.id
        )

    def __ifloordiv__(self, other: Any) -> SyPrimitiveRet:
        return PrimitiveFactory.generate_primitive(
            value=super().__floordiv__(other), id=self.id
        )

    def __itruediv__(self, other: Any) -> SyPrimitiveRet:
        return PrimitiveFactory.generate_primitive(
            value=super().__truediv__(other), id=self.id
        )

    def __imod__(self, other: Any) -> SyPrimitiveRet:
        return PrimitiveFactory.generate_primitive(
            value=super().__mod__(other), id=self.id
        )

    def __ipow__(self, other: Any, modulo: Optional[Any] = None) -> SyPrimitiveRet:
        if modulo:
            return PrimitiveFactory.generate_primitive(
                value=super().__pow__(other, modulo), id=self.id
            )
        return PrimitiveFactory.generate_primitive(
            value=super().__pow__(other), id=self.id
        )

    @property
    def real(self) -> SyPrimitiveRet:
        return PrimitiveFactory.generate_primitive(value=super().real)

    @property
    def imag(self) -> SyPrimitiveRet:
        return PrimitiveFactory.generate_primitive(value=super().imag)

    def conjugate(self) -> SyPrimitiveRet:
        return PrimitiveFactory.generate_primitive(value=super().conjugate())

    def hex(self) -> SyPrimitiveRet:
        return PrimitiveFactory.generate_primitive(value=self.upcast().hex())

    def __hash__(self) -> int:
        return super().__hash__()

    def _object2proto(self) -> Float_PB:
        return Float_PB(
            id=serialize(obj=self.id),
            data=self,
        )

    @staticmethod
    def _proto2object(proto: Float_PB) -> "Float":
        return Float(value=proto.data, id=deserialize(blob=proto.id))

    @staticmethod
    def get_protobuf_schema() -> GeneratedProtocolMessageType:
        return Float_PB



# stdlib
from abc import ABC
from typing import Iterable
from typing import Optional
from typing import Type

# syft relative
from ...logger import debug
from ...logger import traceback_and_raise
from ..common.storeable_object import AbstractStorableObject
from ..common.uid import UID
from .storeable_object import StorableObject


class ObjectStore(ABC):
    """
    ObjectStore is the common interface for all the stores that a Node can handle. This should
    provide a dict-like interface on handling data on a worker. Indexing should be always done
    by using UID objects, while de-indexed value should always be a SerizableObject.
    """

    def __sizeof__(self) -> int:
        """
        Method to return the memory size of the object if possible, if not, it will return __len__.

        Returns:
            int: memory size of the store.
        """
        traceback_and_raise(NotImplementedError)

    def __str__(self) -> str:
        """
        Method to print to first 5 entries of the ObjectStore in a database-like format.

        Returns:
            str: header and first 5 entries of the store in a formatted manner.
        """
        traceback_and_raise(NotImplementedError)

    def __len__(self) -> int:
        """
        Method to return the number of elements in the store.

        Returns:
            int: the number of objects in the store.
        """
        traceback_and_raise(NotImplementedError)

    def keys(self) -> Iterable[UID]:
        """
        Method to return all indexes from the store.

        Returns:
            Iterable[UID]: an iterable containing the keys of the store.
        """
        traceback_and_raise(NotImplementedError)

    def values(self) -> Iterable[StorableObject]:
        """
        Method to return all values from the store.

        Returns:
            Iterable[StorableObject]: an iterable containing the keys of the store.
        """
        traceback_and_raise(NotImplementedError)

    def __contains__(self, key: UID) -> bool:
        """
        Method to check if an UID is present in the store.

        Args:
            key (UID): the UID to be searched.

        Returns:
            bool: if the object is present or not.
        """
        traceback_and_raise(NotImplementedError)

    def __getitem__(self, key: UID) -> StorableObject:
        """
        Method to retrieve an object from the store.

        Args:
            key (UID): The key to be searched for in the store.

        Returns:
            StorableObject: The object associated with the input key.

        Raises:
            ValueError: If the key is not present in the store.
        """
        traceback_and_raise(NotImplementedError)

    def __setitem__(self, key: UID, value: StorableObject) -> None:
        """
        Method to store an object in the store. The difference between this and store is that you
        can you use self.store to store objects at a different key than the one specified in
        the StorableObject.

        This updates the value at the input UID if it exists already.

        Args:
            key (UID): the UID at which to store the StorableObject.
            value (StorableObject): the StorableObject to be stored.
        """
        traceback_and_raise(NotImplementedError)

    def __delitem__(self, key: UID) -> None:
        self.delete(key=key)

    def delete(self, key: UID) -> None:
        """
        We should write custom deletion code so we can check if the item exists
        and then ensure deletion is called at a single place and that full __delitem__
        is used where possible instead of the weaker del ref count.
        Also, care needs to be taken to not capture the deleted item in the Exception
        stack trace so it doesn't keep the item around longer.

        Method to remove an object from the store based on its UID.

        Args:
            key (UID): the key at which to delete the object.

        Raises:
            ValueError: if the key is not present in the store.
        """
        traceback_and_raise(NotImplementedError)

    def clear(self) -> None:
        """
        Clears all storage owned by the store.
        """
        traceback_and_raise(NotImplementedError)

    def get_object(self, key: UID) -> Optional[StorableObject]:
        traceback_and_raise(NotImplementedError)

    def has_object(self) -> None:
        traceback_and_raise(NotImplementedError)

    def store_object(self) -> None:
        traceback_and_raise(NotImplementedError)

    def delete_object(self) -> None:
        traceback_and_raise(NotImplementedError)

    def get_objects_of_type(self, obj_type: Type) -> Iterable[AbstractStorableObject]:
        traceback_and_raise(NotImplementedError)

    @property
    def icon(self) -> str:
        return "🗃️"

    @property
    def pprint(self) -> str:
        output = f"{self.icon} ({self.class_name}) {self.peek}"
        return output

    @property
    def peek(self) -> str:
        output = "{"
        for k in self.keys():
            output += f"\n  > {k} {k.emoji()} => {self[k].pprint}\n"
        output += "}"
        return output

    def post_init(self) -> None:
        debug(f"> Creating {self.pprint}")

    @property
    def class_name(self) -> str:
        return str(self.__class__.__name__)



# -*- coding: utf-8 -*-
"""
Welcome to the syft package! This package is the primary package for PySyft.
This package has two kinds of attributes: submodules and convenience functions.
Submodules are configured in the standard way, but the convenience
functions exist to allow for a convenient `import syft as sy` to then expose
the most-used functionalities directly on syft. Note that this way of importing
PySyft is the strict convention in this codebase. (Do no simply call
`import syft` and then directly use `syft.<method>`.)

The syft module is split into two distinct groups of functionality which we casually refer to
as syft "core" and syft "python". "core" functionality is functionality which is designed
to be universal across all Syft languages (javascript, kotlin, swift, etc.).
Syft "python" includes all functionality which by its very nature cannot be
truly polyglot. Syft "core" functionality includes the following modules:

* :py:mod:`syft.core.node` - APIs for interacting with remote machines you do not directly control.
* :py:mod:`syft.core.message` - APIs for serializing messages sent between Client and Node classes.
* :py:mod:`syft.core.pointer` - Client side API for referring to objects on a Node
* :py:mod:`syft.core.store` - Server side API for referring to object storage on a node (things pointers point to)

Syft "python" functionality includes the following modules:

* :py:mod:`syft.ast` - code generates external library common syntax tree using an allowlist list of methods
* :py:mod:`syft.typecheck` - automatically checks and enforces Python type hints and the exclusive use of kwargs.
* :py:mod:`syft.lib` - uses the ast library to dynamically create remote execution APIs for supported Python libs.

    IMPORTANT: syft.core should be very careful when importing functionality from outside of syft
    core!!! Since we plan to drop syft core down to a language (such as C++ or Rust)
    this can create future complications with lower level languages calling
    higher level ones.

To begin your education in Syft, continue to the :py:mod:`syft.core.node.vm.vm` module...
"""

# stdlib
from pathlib import Path
import sys
from typing import Any
from typing import Dict

# third party
from pkg_resources import DistributionNotFound  # noqa: F401
from pkg_resources import get_distribution  # noqa: F401

# syft absolute
# ASTRACT OBJECT IMPORTS
from syft.core import common  # noqa: F401
from syft.core.common import event_loop  # noqa: F401

# Convenience Methods
from syft.core.common.serde.deserialize import _deserialize as deserialize  # noqa: F401
from syft.core.common.serde.serialize import _serialize as serialize  # noqa: F401
from syft.core.node.common.service.repr_service import ReprMessage  # noqa: F401
from syft.core.node.device.device import Device  # noqa: F401
from syft.core.node.device.device import DeviceClient  # noqa: F401
from syft.core.node.domain.domain import Domain  # noqa: F401
from syft.core.node.domain.domain import DomainClient  # noqa: F401
from syft.core.node.network.network import Network  # noqa: F401
from syft.core.node.network.network import NetworkClient  # noqa: F401

# Convenience Constructors
from syft.core.node.vm.vm import VirtualMachine  # noqa: F401
from syft.core.node.vm.vm import VirtualMachineClient  # noqa: F401
from syft.core.plan.plan import Plan  # noqa: F401
from syft.core.plan.plan_builder import PlanBuilder  # noqa: F401
from syft.core.plan.plan_builder import make_plan  # noqa: F401

# Convenience Functions
from syft.grid.duet import bcolors  # noqa: F401
from syft.grid.duet import duet  # noqa: F401
from syft.grid.duet import join_duet  # noqa: F401
from syft.grid.duet import launch_duet  # noqa: F401

# Convenience Objects
from syft.lib import lib_ast  # noqa: F401
from syft.lib import load  # noqa: F401
from syft.lib import load_lib  # noqa: F401
from syft.lib.torch.module import Module  # noqa: F401

# syft relative
# Package Imports
from . import lib  # noqa: F401
from . import logger  # noqa: F401

# VERSIONING
try:
    # Change here if project is renamed and does not equal the package name
    dist_name = __name__
    __version__ = get_distribution(dist_name).version
except DistributionNotFound:
    __version__ = "unknown"
finally:
    del get_distribution, DistributionNotFound

sys.path.append(str(Path(__file__)))

logger.add(sink=sys.stderr, level="CRITICAL")

# TODO: remove this requirement in pytorch lightning
client_cache: Dict[str, Any] = {}



# third party
import pytest

# syft absolute
import syft as sy


@pytest.mark.vendor(lib="zksk")
def test_statement_zk_proof() -> None:
    vm = sy.VirtualMachine()
    client = vm.get_root_client()

    sy.load("zksk")

    # third party
    from zksk import DLRep
    from zksk import Secret
    from zksk import utils

    num = 2
    seed = 42
    num_sy = sy.lib.python.Int(num)
    seed_sy = sy.lib.python.Int(seed)

    # Setup: Peggy and Victor agree on two group generators.
    G, H = utils.make_generators(num=num, seed=seed)
    # Setup: generate a secret randomizer.
    r = Secret(utils.get_random_num(bits=128))

    # This is Peggy's secret bit.
    top_secret_bit = 1

    # A Pedersen commitment to the secret bit.
    C = top_secret_bit * G + r.value * H

    # Peggy's definition of the proof statement, and proof generation.
    # (The first or-clause corresponds to the secret value 0, and the second to the value 1. Because
    # the real value of the bit is 1, the clause that corresponds to zero is marked as simulated.)
    stmt = DLRep(C, r * H, simulated=True) | DLRep(C - G, r * H)
    zk_proof = stmt.prove()

    # send over the network and get back
    num_ptr = num_sy.send(client)
    seed_prt = seed_sy.send(client)
    c_ptr = C.send(client)
    zk_proof_ptr = zk_proof.send(client)

    num2 = num_ptr.get().upcast()
    seed2 = seed_prt.get().upcast()
    C2 = c_ptr.get()
    zk_proof2 = zk_proof_ptr.get()

    # Setup: get the agreed group generators.
    G, H = utils.make_generators(num=num2, seed=seed2)
    # Setup: define a randomizer with an unknown value.
    r = Secret()

    stmt = DLRep(C2, r * H) | DLRep(C2 - G, r * H)
    assert stmt.verify(zk_proof2)



# type: ignore
# stdlib
from typing import Any
from typing import Callable


def call_func_and_wrap_result(
    func: Callable, wrap_type: Any = None, *args, **kwargs
) -> Any:
    func_res = func(*args, **kwargs)
    return wrap_type(child=func_res)



# stdlib
from typing import Any
from typing import Dict
from typing import Union

# third party
from packaging import version
import torchvision as tv

# syft relative
from ...ast.globals import Globals
from ...logger import critical
from .allowlist import allowlist

TORCHVISION_VERSION = version.parse(tv.__version__)


def get_return_type(support_dict: Union[str, Dict[str, str]]) -> str:
    if isinstance(support_dict, str):
        return support_dict
    else:
        return support_dict["return_type"]


def version_supported(support_dict: Union[str, Dict[str, str]]) -> bool:
    if isinstance(support_dict, str):
        return True
    else:
        if "min_version" not in support_dict.keys():
            return True
        return TORCHVISION_VERSION >= version.parse(support_dict["min_version"])


def create_torchvision_ast(client: Any = None) -> Globals:
    ast = Globals(client)

    # most methods work in all versions and have a single return type
    # for the more complicated ones we pass a dict with keys like return_type and
    # min_version
    for method, return_type_name_or_dict in allowlist.items():
        if version_supported(support_dict=return_type_name_or_dict):
            return_type = get_return_type(support_dict=return_type_name_or_dict)
            ast.add_path(
                path=method,
                framework_reference=tv,
                return_type_name=return_type,
            )
        else:
            critical(
                f"Skipping torchvision.{method} not supported in {TORCHVISION_VERSION}"
            )

    for klass in ast.classes:
        klass.create_pointer_class()
        klass.create_send_method()
        klass.create_storable_object_attr_convenience_methods()
    return ast



# stdlib
import functools
from typing import Any as TypeAny
from typing import List as TypeList
from typing import Tuple as TypeTuple

# third party
import pytorch_lightning as pl

# syft relative
from ...ast import add_classes
from ...ast import add_methods
from ...ast import add_modules
from ...ast.globals import Globals
from ..util import generic_update_ast

LIB_NAME = "pytorch_lightning"
PACKAGE_SUPPORT = {
    "lib": LIB_NAME,
    "torch": {"max_version": "1.8.0"},
    "python": {"max_version": (3, 9, 99)},
}


def create_ast(client: TypeAny) -> Globals:
    ast = Globals(client=client)

    modules: TypeList[TypeTuple[str, TypeAny]] = [("pytorch_lightning", pl)]

    classes: TypeList[TypeTuple[str, str, TypeAny]] = [
        ("pytorch_lightning.Trainer", "pytorch_lightning.Trainer", pl.Trainer),
    ]

    methods: TypeList[TypeTuple[str, str]] = [
        ("pytorch_lightning.Trainer.fit", "syft.lib.python._SyNone"),
        ("pytorch_lightning.Trainer.test", "syft.lib.python._SyNone"),
    ]

    add_modules(ast, modules)
    add_classes(ast, classes)
    add_methods(ast, methods)

    for klass in ast.classes:
        klass.create_pointer_class()
        klass.create_send_method()
        klass.create_storable_object_attr_convenience_methods()

    return ast


update_ast = functools.partial(generic_update_ast, LIB_NAME, create_ast)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/core/node/common/action/run_function_or_constructor.proto
"""Generated protocol buffer code."""
# third party
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


# syft absolute
from syft.proto.core.common import (
    common_object_pb2 as proto_dot_core_dot_common_dot_common__object__pb2,
)
from syft.proto.core.io import address_pb2 as proto_dot_core_dot_io_dot_address__pb2
from syft.proto.core.pointer import (
    pointer_pb2 as proto_dot_core_dot_pointer_dot_pointer__pb2,
)

DESCRIPTOR = _descriptor.FileDescriptor(
    name="proto/core/node/common/action/run_function_or_constructor.proto",
    package="syft.core.node.common.action",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n?proto/core/node/common/action/run_function_or_constructor.proto\x12\x1csyft.core.node.common.action\x1a%proto/core/common/common_object.proto\x1a proto/core/pointer/pointer.proto\x1a\x1bproto/core/io/address.proto"\x8e\x03\n\x1eRunFunctionOrConstructorAction\x12\x0c\n\x04path\x18\x01 \x01(\t\x12(\n\x04\x61rgs\x18\x02 \x03(\x0b\x32\x1a.syft.core.pointer.Pointer\x12X\n\x06kwargs\x18\x03 \x03(\x0b\x32H.syft.core.node.common.action.RunFunctionOrConstructorAction.KwargsEntry\x12-\n\x0eid_at_location\x18\x04 \x01(\x0b\x32\x15.syft.core.common.UID\x12&\n\x07\x61\x64\x64ress\x18\x05 \x01(\x0b\x32\x15.syft.core.io.Address\x12%\n\x06msg_id\x18\x06 \x01(\x0b\x32\x15.syft.core.common.UID\x12\x11\n\tis_static\x18\x07 \x01(\x08\x1aI\n\x0bKwargsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12)\n\x05value\x18\x02 \x01(\x0b\x32\x1a.syft.core.pointer.Pointer:\x02\x38\x01\x62\x06proto3',
    dependencies=[
        proto_dot_core_dot_common_dot_common__object__pb2.DESCRIPTOR,
        proto_dot_core_dot_pointer_dot_pointer__pb2.DESCRIPTOR,
        proto_dot_core_dot_io_dot_address__pb2.DESCRIPTOR,
    ],
)


_RUNFUNCTIONORCONSTRUCTORACTION_KWARGSENTRY = _descriptor.Descriptor(
    name="KwargsEntry",
    full_name="syft.core.node.common.action.RunFunctionOrConstructorAction.KwargsEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="syft.core.node.common.action.RunFunctionOrConstructorAction.KwargsEntry.key",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="syft.core.node.common.action.RunFunctionOrConstructorAction.KwargsEntry.value",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=b"8\001",
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=525,
    serialized_end=598,
)

_RUNFUNCTIONORCONSTRUCTORACTION = _descriptor.Descriptor(
    name="RunFunctionOrConstructorAction",
    full_name="syft.core.node.common.action.RunFunctionOrConstructorAction",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="path",
            full_name="syft.core.node.common.action.RunFunctionOrConstructorAction.path",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="args",
            full_name="syft.core.node.common.action.RunFunctionOrConstructorAction.args",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="kwargs",
            full_name="syft.core.node.common.action.RunFunctionOrConstructorAction.kwargs",
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="id_at_location",
            full_name="syft.core.node.common.action.RunFunctionOrConstructorAction.id_at_location",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.core.node.common.action.RunFunctionOrConstructorAction.address",
            index=4,
            number=5,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.core.node.common.action.RunFunctionOrConstructorAction.msg_id",
            index=5,
            number=6,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="is_static",
            full_name="syft.core.node.common.action.RunFunctionOrConstructorAction.is_static",
            index=6,
            number=7,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[
        _RUNFUNCTIONORCONSTRUCTORACTION_KWARGSENTRY,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=200,
    serialized_end=598,
)

_RUNFUNCTIONORCONSTRUCTORACTION_KWARGSENTRY.fields_by_name[
    "value"
].message_type = proto_dot_core_dot_pointer_dot_pointer__pb2._POINTER
_RUNFUNCTIONORCONSTRUCTORACTION_KWARGSENTRY.containing_type = (
    _RUNFUNCTIONORCONSTRUCTORACTION
)
_RUNFUNCTIONORCONSTRUCTORACTION.fields_by_name[
    "args"
].message_type = proto_dot_core_dot_pointer_dot_pointer__pb2._POINTER
_RUNFUNCTIONORCONSTRUCTORACTION.fields_by_name[
    "kwargs"
].message_type = _RUNFUNCTIONORCONSTRUCTORACTION_KWARGSENTRY
_RUNFUNCTIONORCONSTRUCTORACTION.fields_by_name[
    "id_at_location"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_RUNFUNCTIONORCONSTRUCTORACTION.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_RUNFUNCTIONORCONSTRUCTORACTION.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
DESCRIPTOR.message_types_by_name[
    "RunFunctionOrConstructorAction"
] = _RUNFUNCTIONORCONSTRUCTORACTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

RunFunctionOrConstructorAction = _reflection.GeneratedProtocolMessageType(
    "RunFunctionOrConstructorAction",
    (_message.Message,),
    {
        "KwargsEntry": _reflection.GeneratedProtocolMessageType(
            "KwargsEntry",
            (_message.Message,),
            {
                "DESCRIPTOR": _RUNFUNCTIONORCONSTRUCTORACTION_KWARGSENTRY,
                "__module__": "proto.core.node.common.action.run_function_or_constructor_pb2"
                # @@protoc_insertion_point(class_scope:syft.core.node.common.action.RunFunctionOrConstructorAction.KwargsEntry)
            },
        ),
        "DESCRIPTOR": _RUNFUNCTIONORCONSTRUCTORACTION,
        "__module__": "proto.core.node.common.action.run_function_or_constructor_pb2"
        # @@protoc_insertion_point(class_scope:syft.core.node.common.action.RunFunctionOrConstructorAction)
    },
)
_sym_db.RegisterMessage(RunFunctionOrConstructorAction)
_sym_db.RegisterMessage(RunFunctionOrConstructorAction.KwargsEntry)


_RUNFUNCTIONORCONSTRUCTORACTION_KWARGSENTRY._options = None
# @@protoc_insertion_point(module_scope)



# stdlib
from typing import Set

# syft relative
from ..location import Location
from .group import LocationGroup


class RegistryBackedLocationGroup(LocationGroup):

    """This is a location group where members of the group is
    determined by an official owner who has an official list. The
    best and most practical example of this is a Node and its group
    of children (or grandchildren, etc.). Sending messages to this
    group requires sending a message to its owner and asking it
    to redistribute to the group. However, we can still maintain a list
    of workers which we believe to be in the group which allows us
    to logically reason about the group itself."""

    def __init__(self, group_owner: Location, known_group_members: Set[Location]):
        super().__init__(known_group_members=known_group_members)
        self.group_owner = group_owner



# stdlib
from typing import Any
from typing import Dict

# syft absolute
import syft as sy
from syft import serialize
from syft.core.io.address import Address
from syft.grid.messages.association_messages import DeleteAssociationRequestMessage
from syft.grid.messages.association_messages import DeleteAssociationRequestResponse
from syft.grid.messages.association_messages import GetAssociationRequestMessage
from syft.grid.messages.association_messages import GetAssociationRequestResponse
from syft.grid.messages.association_messages import GetAssociationRequestsMessage
from syft.grid.messages.association_messages import GetAssociationRequestsResponse
from syft.grid.messages.association_messages import ReceiveAssociationRequestMessage
from syft.grid.messages.association_messages import ReceiveAssociationRequestResponse
from syft.grid.messages.association_messages import RespondAssociationRequestMessage
from syft.grid.messages.association_messages import RespondAssociationRequestResponse
from syft.grid.messages.association_messages import SendAssociationRequestMessage
from syft.grid.messages.association_messages import SendAssociationRequestResponse


def test_association_request_message_serde(node: sy.VirtualMachine) -> None:
    target = Address(name="Alice")

    request_content = {"domain-name": "My-Domain", "domain-address": "http://url:5000"}
    msg = SendAssociationRequestMessage(
        address=target,
        content=request_content,
        reply_to=node.address,
    )

    blob = serialize(msg)
    msg2 = sy.deserialize(blob=blob)

    assert msg.id == msg2.id
    assert msg.address == target
    assert msg.content == msg2.content
    assert msg == msg2


def test_association_request_response_serde() -> None:
    target = Address(name="Alice")

    request_content = {"msg": "Association Request Accepted status_codefully!"}
    msg = SendAssociationRequestResponse(
        address=target,
        status_code=200,
        content=request_content,
    )

    blob = serialize(msg)
    msg2 = sy.deserialize(blob=blob)

    assert msg.id == msg2.id
    assert msg.address == target
    assert msg.content == msg2.content
    assert msg == msg2


def test_receive_request_message_serde(node: sy.VirtualMachine) -> None:
    target = Address(name="Alice")

    request_content: Dict[Any, Any] = {}
    msg = ReceiveAssociationRequestMessage(
        address=target,
        content=request_content,
        reply_to=node.address,
    )

    blob = serialize(msg)
    msg2 = sy.deserialize(blob=blob)

    assert msg.id == msg2.id
    assert msg.address == target
    assert msg.content == msg2.content
    assert msg == msg2


def test_receive_request_response_serde() -> None:
    target = Address(name="Alice")

    content: Dict[Any, Any] = {}
    msg = ReceiveAssociationRequestResponse(
        status_code=200,
        address=target,
        content=content,
    )

    blob = serialize(msg)
    msg2 = sy.deserialize(blob=blob)

    assert msg.id == msg2.id
    assert msg.address == target
    assert msg.content == msg2.content
    assert msg == msg2


def test_respond_association_request_message_serde(node: sy.VirtualMachine) -> None:
    target = Address(name="Alice")

    content = {"association_request_id": "87564178", "status": "accept"}
    msg = RespondAssociationRequestMessage(
        address=target,
        content=content,
        reply_to=node.address,
    )

    blob = serialize(msg)
    msg2 = sy.deserialize(blob=blob)

    assert msg.id == msg2.id
    assert msg.address == target
    assert msg.content == msg2.content
    assert msg == msg2


def test_respond_association_request_response_serde() -> None:
    target = Address(name="Alice")

    request_content = {"msg": "Response registered status_codefully!"}
    msg = RespondAssociationRequestResponse(
        address=target,
        status_code=200,
        content=request_content,
    )

    blob = serialize(msg)
    msg2 = sy.deserialize(blob=blob)

    assert msg.id == msg2.id
    assert msg.address == target
    assert msg.content == msg2.content
    assert msg == msg2


def test_delete_association_request_message_serde(node: sy.VirtualMachine) -> None:
    target = Address(name="Alice")

    content = {"association_request_id": "21656565"}
    msg = DeleteAssociationRequestMessage(
        address=target,
        content=content,
        reply_to=node.address,
    )

    blob = serialize(msg)
    msg2 = sy.deserialize(blob=blob)

    assert msg.id == msg2.id
    assert msg.address == target
    assert msg.content == msg2.content
    assert msg == msg2


def test_delete_association_request_response_serde() -> None:
    target = Address(name="Alice")

    content = {"msg": "Association Request deleted status_codefully!"}
    msg = DeleteAssociationRequestResponse(
        address=target,
        status_code=200,
        content=content,
    )

    blob = serialize(msg)
    msg2 = sy.deserialize(blob=blob)

    assert msg.id == msg2.id
    assert msg.address == target
    assert msg.content == msg2.content
    assert msg == msg2


def test_get_association_request_message_serde(node: sy.VirtualMachine) -> None:
    target = Address(name="Alice")

    content = {"association_request_id": "87564178"}
    msg = GetAssociationRequestMessage(
        address=target,
        content=content,
        reply_to=node.address,
    )

    blob = serialize(msg)
    msg2 = sy.deserialize(blob=blob)

    assert msg.id == msg2.id
    assert msg.address == target
    assert msg.content == msg2.content
    assert msg == msg2


def test_get_association_request_response_serde() -> None:
    target = Address(name="Alice")

    request_content = {
        "entity": "OpenMined",
        "entity-type": "Network",
        "status": "pending",
        "date": "05/12/2022",
    }
    msg = GetAssociationRequestResponse(
        address=target,
        status_code=200,
        content=request_content,
    )

    blob = serialize(msg)
    msg2 = sy.deserialize(blob=blob)

    assert msg.id == msg2.id
    assert msg.address == target
    assert msg.content == msg2.content
    assert msg == msg2


def test_get_all_association_request_message_serde(node: sy.VirtualMachine) -> None:
    target = Address(name="Alice")

    content: Dict[Any, Any] = {}
    msg = GetAssociationRequestsMessage(
        address=target,
        content=content,
        reply_to=node.address,
    )

    blob = serialize(msg)
    msg2 = sy.deserialize(blob=blob)

    assert msg.id == msg2.id
    assert msg.address == target
    assert msg.content == msg2.content
    assert msg == msg2


def test_get_all_association_request_response_serde() -> None:
    target = Address(name="Alice")

    request_content = {
        "association-requests": [
            {
                "entity": "OpenMined",
                "entity-type": "Network",
                "status": "pending",
                "date": "05/12/2022",
            },
            {
                "entity": "Hospital-A",
                "entity-type": "Domain",
                "status": "pending",
                "date": "09/10/2022",
            },
            {
                "entity": "OpenMined",
                "entity-type": "Network",
                "status": "pending",
                "date": "07/11/2022",
            },
        ]
    }
    msg = GetAssociationRequestsResponse(
        address=target,
        status_code=200,
        content=request_content,
    )

    blob = serialize(msg)
    msg2 = sy.deserialize(blob=blob)

    assert msg.id == msg2.id
    assert msg.address == target
    assert msg.content == msg2.content
    assert msg == msg2



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/core/node/common/action/exception_action.proto
"""Generated protocol buffer code."""
# third party
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


# syft absolute
from syft.proto.core.common import (
    common_object_pb2 as proto_dot_core_dot_common_dot_common__object__pb2,
)
from syft.proto.core.io import address_pb2 as proto_dot_core_dot_io_dot_address__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name="proto/core/node/common/action/exception_action.proto",
    package="syft.core.node.common.service",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n4proto/core/node/common/action/exception_action.proto\x12\x1dsyft.core.node.common.service\x1a%proto/core/common/common_object.proto\x1a\x1bproto/core/io/address.proto"\xc9\x01\n\x10\x45xceptionMessage\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12&\n\x07\x61\x64\x64ress\x18\x02 \x01(\x0b\x32\x15.syft.core.io.Address\x12\x37\n\x18msg_id_causing_exception\x18\x03 \x01(\x0b\x32\x15.syft.core.common.UID\x12\x16\n\x0e\x65xception_type\x18\x04 \x01(\t\x12\x15\n\rexception_msg\x18\x05 \x01(\tb\x06proto3',
    dependencies=[
        proto_dot_core_dot_common_dot_common__object__pb2.DESCRIPTOR,
        proto_dot_core_dot_io_dot_address__pb2.DESCRIPTOR,
    ],
)


_EXCEPTIONMESSAGE = _descriptor.Descriptor(
    name="ExceptionMessage",
    full_name="syft.core.node.common.service.ExceptionMessage",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.core.node.common.service.ExceptionMessage.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.core.node.common.service.ExceptionMessage.address",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="msg_id_causing_exception",
            full_name="syft.core.node.common.service.ExceptionMessage.msg_id_causing_exception",
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="exception_type",
            full_name="syft.core.node.common.service.ExceptionMessage.exception_type",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="exception_msg",
            full_name="syft.core.node.common.service.ExceptionMessage.exception_msg",
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=156,
    serialized_end=357,
)

_EXCEPTIONMESSAGE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_EXCEPTIONMESSAGE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_EXCEPTIONMESSAGE.fields_by_name[
    "msg_id_causing_exception"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
DESCRIPTOR.message_types_by_name["ExceptionMessage"] = _EXCEPTIONMESSAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ExceptionMessage = _reflection.GeneratedProtocolMessageType(
    "ExceptionMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _EXCEPTIONMESSAGE,
        "__module__": "proto.core.node.common.action.exception_action_pb2"
        # @@protoc_insertion_point(class_scope:syft.core.node.common.service.ExceptionMessage)
    },
)
_sym_db.RegisterMessage(ExceptionMessage)


# @@protoc_insertion_point(module_scope)



# syft absolute
from syft.lib.python.int import Int
from syft.lib.python.list import List

sy_list = List([Int(1), Int(2), Int(3)])
other_list = List([Int(4)])
other = Int(4)
py_list = [1, 2, 3]


def test_id_contains():
    sy_res = sy_list.__contains__(other)
    py_res = py_list.__contains__(other)
    assert py_res == sy_res
    assert sy_res.id != sy_list.id


def test_id_delitem():
    tmp_list = List([1, 2, 3])
    id = tmp_list.id
    del tmp_list[0]
    assert id == tmp_list.id


def test_id_eq():
    sy_res = sy_list == other_list
    py_res = py_list == other_list
    assert py_res == sy_res
    assert sy_res.id != sy_list.id


def test_id_ge():
    sy_res = sy_list >= other_list
    py_res = py_list >= other_list
    assert py_res == sy_res
    assert sy_res.id != sy_list.id


def test_id_gt():
    sy_res = sy_list > other_list
    py_res = py_list > other_list
    assert py_res == sy_res
    assert sy_res.id != sy_list.id


def test_id_iadd():
    sy_res = sy_list.__iadd__(other_list)
    py_res = py_list.__iadd__(other_list)
    assert py_res == sy_res
    assert sy_res.id == sy_list.id


def test_id_imul():
    sy_res = sy_list.__imul__(other)
    py_res = py_list.__imul__(other)
    assert py_res == sy_res
    assert sy_res.id == sy_list.id


def test_id_le():
    sy_res = sy_list <= other_list
    py_res = py_list <= other_list
    assert py_res == sy_res
    assert sy_res.id != sy_list.id


def test_id_len():
    sy_res = sy_list.__len__()
    py_res = py_list.__len__()
    assert py_res == sy_res
    assert sy_res.id != sy_list.id


def test_id_lt():
    sy_res = sy_list < other_list
    py_res = py_list < other_list
    assert py_res == sy_res
    assert sy_res.id != sy_list.id


def test_id_mul():
    sy_res = sy_list.__mul__(other)
    py_res = py_list.__mul__(other)
    assert py_res == sy_res
    assert sy_res.id != sy_list.id


def test_id_ne():
    sy_res = sy_list != other
    py_res = py_list != other
    assert py_res == sy_res
    assert sy_res.id != sy_list.id


def test_id_radd():
    sy_res = sy_list.__radd__(other_list)
    assert sy_res.id != sy_list.id


def test_id_rmul():
    sy_res = sy_list.__rmul__(other)
    assert sy_res.id
    assert sy_res.id != sy_list.id


def test_id_sizeof():
    sy_res = sy_list.__sizeof__()
    assert sy_res.id
    assert sy_res.id != sy_list.id


def test_id_append():
    tmp_list = List([1, 2, 3])
    tmp_id = tmp_list.id
    tmp_list.append(4)
    assert tmp_id == tmp_list.id


def test_id_clear():
    tmp_list = List([1, 2, 3])
    tmp_id = tmp_list.id
    tmp_list.clear()
    assert tmp_id == tmp_list.id


def test_id_copy():
    sy_res = sy_list.copy()
    py_res = py_list.copy()
    assert py_res == sy_res
    assert sy_res.id != sy_list.id


def test_id_count():
    sy_res = sy_list.count(other)
    py_res = py_list.count(other)
    assert py_res == sy_res
    assert sy_res.id != sy_list.id


def test_id_te():
    tmp_list = List([1, 2, 3])
    tmp_id = tmp_list.id
    tmp_list.extend(List([1, 2, 3]))
    assert tmp_list.id == tmp_id


def test_id_pop():
    tmp_list = List([1, 2, 3])
    tmp_id = tmp_list.id
    tmp_list.pop(0)
    assert tmp_list.id == tmp_id


def test_id_remove():
    tmp_list = List([1, 2, 3])
    tmp_id = tmp_list.id
    tmp_list.remove(3)
    assert tmp_list.id == tmp_id


def test_id_sort():
    tmp_list = List([1, 2, 3])
    tmp_id = tmp_list.id
    tmp_list.sort()
    assert tmp_list.id == tmp_id



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/core/io/connection.proto
"""Generated protocol buffer code."""
# third party
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


# syft absolute
from syft.proto.core.io import address_pb2 as proto_dot_core_dot_io_dot_address__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name="proto/core/io/connection.proto",
    package="syft.core.io",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x1eproto/core/io/connection.proto\x12\x0csyft.core.io\x1a\x1bproto/core/io/address.proto">\n\x17VirtualServerConnection\x12#\n\x04node\x18\x01 \x01(\x0b\x32\x15.syft.core.io.Address"P\n\x17VirtualClientConnection\x12\x35\n\x06server\x18\x01 \x01(\x0b\x32%.syft.core.io.VirtualServerConnectionb\x06proto3',
    dependencies=[
        proto_dot_core_dot_io_dot_address__pb2.DESCRIPTOR,
    ],
)


_VIRTUALSERVERCONNECTION = _descriptor.Descriptor(
    name="VirtualServerConnection",
    full_name="syft.core.io.VirtualServerConnection",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="node",
            full_name="syft.core.io.VirtualServerConnection.node",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=77,
    serialized_end=139,
)


_VIRTUALCLIENTCONNECTION = _descriptor.Descriptor(
    name="VirtualClientConnection",
    full_name="syft.core.io.VirtualClientConnection",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="server",
            full_name="syft.core.io.VirtualClientConnection.server",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=141,
    serialized_end=221,
)

_VIRTUALSERVERCONNECTION.fields_by_name[
    "node"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_VIRTUALCLIENTCONNECTION.fields_by_name[
    "server"
].message_type = _VIRTUALSERVERCONNECTION
DESCRIPTOR.message_types_by_name["VirtualServerConnection"] = _VIRTUALSERVERCONNECTION
DESCRIPTOR.message_types_by_name["VirtualClientConnection"] = _VIRTUALCLIENTCONNECTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

VirtualServerConnection = _reflection.GeneratedProtocolMessageType(
    "VirtualServerConnection",
    (_message.Message,),
    {
        "DESCRIPTOR": _VIRTUALSERVERCONNECTION,
        "__module__": "proto.core.io.connection_pb2"
        # @@protoc_insertion_point(class_scope:syft.core.io.VirtualServerConnection)
    },
)
_sym_db.RegisterMessage(VirtualServerConnection)

VirtualClientConnection = _reflection.GeneratedProtocolMessageType(
    "VirtualClientConnection",
    (_message.Message,),
    {
        "DESCRIPTOR": _VIRTUALCLIENTCONNECTION,
        "__module__": "proto.core.io.connection_pb2"
        # @@protoc_insertion_point(class_scope:syft.core.io.VirtualClientConnection)
    },
)
_sym_db.RegisterMessage(VirtualClientConnection)


# @@protoc_insertion_point(module_scope)



# stdlib
from typing import Any as TypeAny
from typing import Callable
from typing import Dict as TypeDict
from typing import List as ListType
from typing import Optional

# syft relative
from ..federated import JSONDict
from ..logger import traceback_and_raise
from .model_centric_fl_base import GridError
from .model_centric_fl_worker import ModelCentricFLWorker


class EventEmitter:
    def __init__(self) -> None:
        self.listeners: TypeDict[str, ListType[Callable]] = {}

    def add_listener(self, event_name: str, fn: Callable) -> None:
        if event_name not in self.listeners:
            self.listeners[event_name] = []
        self.listeners[event_name].append(fn)

    def trigger(self, event_name: str, *args: TypeAny, **kwargs: TypeAny) -> None:
        if event_name in self.listeners:
            for fn in self.listeners[event_name]:
                fn(*args, **kwargs)


class FLJob(EventEmitter):
    EVENT_ACCEPTED: str = "accepted"
    EVENT_REJECTED: str = "rejected"
    EVENT_ERROR: str = "error"

    def __init__(
        self,
        worker_id: str,
        grid_worker: ModelCentricFLWorker,
        model_name: str,
        model_version: str,
    ) -> None:
        super().__init__()
        self.worker_id = worker_id
        self.grid_worker = grid_worker
        self.model_name = model_name
        self.model_version = model_version
        self.plan_type = ModelCentricFLWorker.PLAN_TYPE_LIST

        self.model: Optional[ListType[TypeAny]] = None
        self.plans: JSONDict = {}
        self.cycle_params: JSONDict = {}
        self.client_config: JSONDict = {}

    def _init_cycle(self, cycle_params: JSONDict) -> None:
        self.cycle_params = cycle_params
        self.client_config = cycle_params["client_config"]

        request_key = cycle_params["request_key"]

        # Load model
        try:
            self.model = self.grid_worker.get_model(
                self.worker_id, request_key, cycle_params["model_id"]
            )
            if not self.model:
                traceback_and_raise(
                    ValueError(f"Model is not valid so {self} can't run")
                )
        except Exception as e:
            traceback_and_raise(
                ValueError(f"Failed to fetch model during {self}._init_cycle. {e}")
            )

        # Load plans
        for plan_name, plan_id in cycle_params["plans"].items():
            self.plans[plan_name] = self.grid_worker.get_plan(
                self.worker_id,
                request_key,
                plan_id,
                self.plan_type,
            )

    def start(self) -> None:
        try:
            speed_info = self.grid_worker.get_connection_speed(self.worker_id)
            cycle_request_response = self.grid_worker.cycle_request(
                worker_id=self.worker_id,
                model_name=self.model_name,
                model_version=self.model_version,
                speed_info=speed_info,
            )

            cycle_params = cycle_request_response["data"]

            if cycle_params["status"] == ModelCentricFLWorker.CYCLE_STATUS_ACCEPTED:
                self._init_cycle(cycle_params)
                self.trigger(self.EVENT_ACCEPTED, self)
            elif cycle_params["status"] == ModelCentricFLWorker.CYCLE_STATUS_REJECTED:
                timeout = cycle_params.get("timeout")
                if timeout is not None:
                    timeout = timeout(int)
                self.trigger(self.EVENT_REJECTED, self, timeout)
        except GridError as e:
            self.trigger(
                self.EVENT_ERROR,
                self,
                f"Grid communication error: {e.error}",
            )

    def report(self, updated_model_params: ListType) -> JSONDict:
        # Calc params diff
        if not self.model:
            traceback_and_raise(
                ValueError(f"Model is not valid so {self} can't report")
            )
        diff_params: ListType[TypeAny] = [
            o_p - updated_model_params[i] for i, o_p in enumerate(self.model)
        ]

        return self.grid_worker.report(
            worker_id=self.worker_id,
            request_key=self.cycle_params["request_key"],
            diff=diff_params,
        )



# stdlib
from typing import Any
from typing import Optional
import uuid
from uuid import UUID as uuid_type

# third party
from google.protobuf.reflection import GeneratedProtocolMessageType

# syft relative
from ...logger import critical
from ...logger import traceback_and_raise
from ...proto.core.common.common_object_pb2 import UID as UID_PB
from ..common.serde.serializable import Serializable
from ..common.serde.serializable import bind_protobuf


@bind_protobuf
class UID(Serializable):
    """A unique ID for every Syft object.

    This object creates a unique ID for every object in the Syft
    ecosystem. This ID is guaranteed to be unique for the node on
    which it is initialized and is very likely to be unique across
    the whole ecosystem (because it is long and randomly generated).

    Nearly all objects within Syft subclass from this object because
    nearly all objects need to have a unique ID. The only major
    exception a the time of writing is the Client object because it
    just points to another object which itself has an id.

    There is no other way in Syft to create an ID for any object.

    """

    value: uuid_type

    def __init__(self, value: Optional[uuid_type] = None):
        """Initializes the internal id using the uuid package.

        This initializes the object. Normal use for this object is
        to initialize the constructor with value==None because you
        want to initialize with a novel ID. The only major exception
        is deserialization, wherein a UID object is created with a
        specific id value.

        :param value: if you want to initialize an object with a specific UID, pass it
                      in here. This is normally only used during deserialization.
        :type value: uuid.uuid4, optional
        :return: returns the initialized object
        :rtype: UID

        .. code-block:: python

            from syft.core.common.uid import UID
            my_id = UID()
        """
        # checks to make sure you've set a proto_type
        super().__init__()

        # if value is not set - create a novel and unique ID.
        if value is None:

            # for more info on how this UUID is generated:
            # https://docs.python.org/3/library/uuid.html
            value = uuid.uuid4()

        # save the ID's value. Note that this saves the uuid value
        # itself instead of saving the
        self.value = value

    @staticmethod
    def from_string(value: str) -> "UID":
        try:
            return UID(value=uuid.UUID(value))
        except Exception as e:
            critical(f"Unable to convert {value} to UUID. {e}")
            traceback_and_raise(e)

    def __hash__(self) -> int:
        """Hashes the UID for use in dictionaries and sets

        A very common use of UID objects is as a key in a dictionary
        or database. The object must be able to be hashed in order to
        be used in this way. We take the 128-bit int representation of the
        value.

        :return: returns a hash of the object
        :rtype: int

        .. note::
            Note that this probably gets further hashed into a shorter
            representation for most python data-structures.

        .. note::
            Note that we assume that any collisions will be very rare and
            detected by the ObjectStore class in Syft.
        """

        return self.value.int

    def __eq__(self, other: Any) -> bool:
        """Checks to see if two UIDs are the same using the internal object

        This checks to see whether this UID is equal to another UID by
        comparing whether they have the same .value objects. These objects
        come with their own __eq__ function which we assume to be correct.

        :param other: this is the other ID to be compared with
        :type other: Any (note this must be Any or __eq__ fails on other types)
        :return: returns True/False based on whether the objects are the same
        :rtype: bool
        """

        try:
            return self.value == other.value
        except Exception:
            return False

    def __repr__(self) -> str:
        """Returns a human-readable version of the ID

        Return a human-readable representation of the UID with brackets
        so that it can be easily spotted when nested inside of the human-
        readable representations of other objects."""

        no_dash = str(self.value).replace("-", "")
        return f"<{type(self).__name__}: {no_dash}>"

    def char_emoji(self, hex_chars: str) -> str:
        base = ord("\U0001F642")
        hex_base = ord("0")
        code = 0
        for char in hex_chars:
            offset = ord(char)
            code += offset - hex_base
        return chr(base + code)

    def string_emoji(self, string: str, length: int, chunk: int) -> str:
        output = []
        part = string[-length:]
        while len(part) > 0:
            part, end = part[:-chunk], part[-chunk:]
            output.append(self.char_emoji(hex_chars=end))
        return "".join(output)

    def emoji(self) -> str:
        return f"<UID:{self.string_emoji(string=str(self.value), length=8, chunk=4)}>"

    def repr_short(self) -> str:
        """Returns a SHORT human-readable version of the ID

        Return a SHORT human-readable version of the ID which
        makes it print nicer when embedded (often alongside other
        UID objects) within other object __repr__ methods."""

        return f"..{str(self.value)[-5:]}"

    def _object2proto(self) -> UID_PB:
        """Returns a protobuf serialization of self.

        As a requirement of all objects which inherit from Serializable,
        this method transforms the current object into the corresponding
        Protobuf object so that it can be further serialized.

        :return: returns a protobuf object
        :rtype: ProtoUID

        .. note::
            This method is purely an internal method. Please use serialize(object) or one of
            the other public serialization methods if you wish to serialize an
            object.
        """

        return UID_PB(value=self.value.bytes)

    @staticmethod
    def _proto2object(proto: UID_PB) -> "UID":
        """Creates a UID from a protobuf

        As a requirement of all objects which inherit from Serializable,
        this method transforms a protobuf object into an instance of this class.

        :return: returns an instance of UID
        :rtype: UID

        .. note::
            This method is purely an internal method. Please use syft.deserialize()
            if you wish to deserialize an object.
        """
        return UID(value=uuid.UUID(bytes=proto.value))

    @staticmethod
    def get_protobuf_schema() -> GeneratedProtocolMessageType:
        """Return the type of protobuf object which stores a class of this type

        As a part of serialization and deserialization, we need the ability to
        lookup the protobuf object type directly from the object type. This
        static method allows us to do this.

        Importantly, this method is also used to create the reverse lookup ability within
        the metaclass of Serializable. In the metaclass, it calls this method and then
        it takes whatever type is returned from this method and adds an attribute to it
        with the type of this class attached to it. See the MetaSerializable class for details.

        :return: the type of protobuf object which corresponds to this class.
        :rtype: GeneratedProtocolMessageType

        """
        return UID_PB



# stdlib
from typing import Any as TypeAny
from typing import List as TypeList
from typing import Tuple as TypeTuple

# syft relative
from ...ast import add_classes
from ...ast import add_methods
from ...ast import add_modules
from ...ast.globals import Globals
from ...core.plan.plan import Plan
from ...core.plan.translation.torchscript.plan import PlanTorchscript


def create_plan_ast(client: TypeAny = None) -> Globals:
    """Adds plan classes to the ast

    Args:
        client: client. Defaults to None.

    Returns:
        Globals: plan ast
    """
    ast = Globals(client)

    modules = [
        "syft",
        "syft.core",
        "syft.core",
        "syft.core.plan",
        "syft.core.plan.plan",
        "syft.core.plan.translation",
        "syft.core.plan.translation.torchscript",
        "syft.core.plan.translation.torchscript.plan",
    ]

    classes: TypeList[TypeTuple[str, str, TypeAny]] = [
        (
            "syft.core.plan.Plan",
            "syft.core.plan.Plan",
            Plan,
        ),
        (
            "syft.core.plan.translation.torchscript.PlanTorchscript",
            "syft.core.plan.translation.torchscript.PlanTorchscript",
            PlanTorchscript,
        ),
    ]

    methods: TypeList[TypeTuple[str, str]] = [
        ("syft.core.plan.Plan.__call__", "syft.lib.python.List"),
    ]

    add_modules(ast, modules)
    add_classes(ast, classes)
    add_methods(ast, methods)

    for klass in ast.classes:
        klass.create_pointer_class()
        klass.create_send_method()
        klass.create_storable_object_attr_convenience_methods()

    return ast



# -*- coding: utf-8 -*-
"""
    Dummy conftest.py for syft.
    If you don't know what this is for, just leave it empty.
    Read more about conftest.py under:
    https://pytest.org/latest/plugins.html
"""

# stdlib
import logging
from typing import Any as TypeAny
from typing import Dict as TypeDict
from typing import Generator
from typing import List as TypeList

# third party
import _pytest
import pytest

# syft absolute
import syft as sy
from syft import logger
from syft.lib import VendorLibraryImportException
from syft.lib import _load_lib
from syft.lib import vendor_requirements_available

logger.remove()


@pytest.fixture
def caplog(caplog: _pytest.logging.LogCaptureFixture) -> Generator:
    class PropogateHandler(logging.Handler):
        def emit(self, record: logging.LogRecord) -> None:
            logging.getLogger(record.name).handle(record)

    logger.add(PropogateHandler())
    yield caplog
    logger.remove()


def pytest_addoption(parser: _pytest.config.argparsing.Parser) -> None:
    parser.addoption(
        "--runslow", action="store_true", default=False, help="run slow tests"
    )


def pytest_configure(config: _pytest.config.Config) -> None:
    config.addinivalue_line("markers", "slow: mark test as slow to run")
    config.addinivalue_line("markers", "fast: mark test as fast to run")
    config.addinivalue_line("markers", "all: all tests")
    config.addinivalue_line("markers", "asyncio: mark test as asyncio")
    config.addinivalue_line("markers", "vendor: mark test as vendor library")
    config.addinivalue_line("markers", "libs: runs valid vendor tests")
    config.addinivalue_line("markers", "benchmark: runs benchmark tests")
    config.addinivalue_line("markers", "torch: runs torch tests")
    config.addinivalue_line("markers", "duet: runs duet notebook integration tests")
    config.addinivalue_line("markers", "grid: runs grid tests")


def pytest_collection_modifyitems(
    config: _pytest.config.Config, items: TypeList[TypeAny]
) -> None:
    # $ pytest -m fast for the fast tests
    # $ pytest -m slow for the slow tests
    # $ pytest -m all for all the tests
    # $ pytest -m libs for the vendor tests
    slow_tests = pytest.mark.slow
    fast_tests = pytest.mark.fast
    duet_tests = pytest.mark.duet
    grid_tests = pytest.mark.grid
    all_tests = pytest.mark.all

    # dynamically filtered vendor lib tests
    # there isn't any way to remove "vendor" so the only way to filter
    # these tests is to add a different tag called "libs" and then run
    # the tests against that dynamic keyword
    vendor_tests = pytest.mark.libs  # note libs != vendor
    loaded_libs: TypeDict[str, bool] = {}
    vendor_skip = pytest.mark.skip(reason="vendor requirements not met")
    for item in items:
        if item.location[0].startswith("PyGrid"):
            # Ignore if PyGrid folder checked out in main dir
            continue

        if "grid" in item.keywords:
            item.add_marker(grid_tests)
            continue
        # mark with: pytest.mark.vendor
        # run with: pytest -m libs -n auto 0
        if "vendor" in item.keywords:
            vendor_requirements = item.own_markers[0].kwargs

            # try to load the lib first and if it fails just skip
            if "lib" in vendor_requirements:
                lib_name = vendor_requirements["lib"]
                if lib_name not in loaded_libs:
                    try:
                        _load_lib(lib=lib_name)
                        loaded_libs[lib_name] = True
                    except Exception as e:
                        print(f"Failed to load {lib_name}. {e}")
                        loaded_libs[lib_name] = False
                if not loaded_libs[lib_name]:
                    item.add_marker(vendor_skip)
                    continue

            try:
                # test the vendor requirements of the specific test if the library
                # was loaded successfully
                if vendor_requirements_available(
                    vendor_requirements=vendor_requirements
                ):
                    if item.location[0].startswith("tests/syft/notebooks"):
                        item.add_marker(duet_tests)
                    else:
                        item.add_marker(vendor_tests)
                    item.add_marker(all_tests)

            except VendorLibraryImportException as e:
                print(e)
            except Exception as e:
                print(f"Unable to check vendor library: {vendor_requirements}. {e}")
            continue

        if "benchmark" in item.keywords:
            continue

        if "torch" in item.keywords:
            item.add_marker(all_tests)
            continue

        item.add_marker(all_tests)
        if "slow" in item.keywords:
            item.add_marker(slow_tests)
        else:
            if item.location[0].startswith("tests/syft/notebooks"):
                item.add_marker(duet_tests)
                continue
            # fast is the default catch all
            item.add_marker(fast_tests)


@pytest.fixture(scope="session")
def node() -> sy.VirtualMachine:
    return sy.VirtualMachine(name="Bob")


@pytest.fixture(autouse=True)
def node_store(node: sy.VirtualMachine) -> None:
    node.store.clear()


@pytest.fixture(scope="session")
def client(node: sy.VirtualMachine) -> sy.VirtualMachineClient:
    return node.get_client()


@pytest.fixture(scope="session")
def root_client(node: sy.VirtualMachine) -> sy.VirtualMachineClient:
    return node.get_root_client()



# stdlib
from typing import Optional

# third party
from google.protobuf.reflection import GeneratedProtocolMessageType
from nacl.signing import VerifyKey

# syft relative
from ..... import serialize
from .....logger import critical
from .....proto.core.node.common.action.garbage_collect_object_pb2 import (
    GarbageCollectObjectAction as GarbageCollectObjectAction_PB,
)
from ....common.serde.deserialize import _deserialize
from ....common.serde.serializable import bind_protobuf
from ....common.uid import UID
from ....io.address import Address
from ...abstract.node import AbstractNode
from .common import EventualActionWithoutReply


@bind_protobuf
class GarbageCollectObjectAction(EventualActionWithoutReply):
    def __init__(
        self, id_at_location: UID, address: Address, msg_id: Optional[UID] = None
    ):
        super().__init__(address=address, msg_id=msg_id)
        self.id_at_location = id_at_location

    def execute_action(self, node: AbstractNode, verify_key: VerifyKey) -> None:
        try:
            node.store.delete(key=self.id_at_location)
        except Exception as e:
            critical(
                "> GarbageCollectObjectAction deletion exception "
                + f"{self.id_at_location} {e}"
            )

    def _object2proto(self) -> GarbageCollectObjectAction_PB:

        id_pb = serialize(self.id_at_location)
        addr = serialize(self.address)

        return GarbageCollectObjectAction_PB(
            id_at_location=id_pb,
            address=addr,
        )

    @staticmethod
    def _proto2object(
        proto: GarbageCollectObjectAction_PB,
    ) -> "GarbageCollectObjectAction":

        id_at_location = _deserialize(blob=proto.id_at_location)
        addr = _deserialize(blob=proto.address)

        return GarbageCollectObjectAction(
            id_at_location=id_at_location,
            address=addr,
        )

    @staticmethod
    def get_protobuf_schema() -> GeneratedProtocolMessageType:
        return GarbageCollectObjectAction_PB



# stdlib
from typing import Any

# third party
import pytest

# syft absolute
import syft as sy


def get_permission(obj: Any, node: sy.VirtualMachine) -> None:
    remote_obj = node.store[obj.id_at_location]
    remote_obj.read_permissions[node.verify_key] = obj.id_at_location


inputs = {
    "__add__": [["str_1"], [""]],
    "__contains__": [[""], ["tu"], ["AND"], ["\t"]],
    "__eq__:": [["george"], ["test"]],
    "__ge__": [["test"], ["aaaatest"]],
    "__getitem__": [[0], [2]],
    "__gt__": [["test"], ["aaaatest"]],
    "__le__": [["test"], ["aaaatest"]],
    "__len__": [[]],
    "__lt__": [["test"], ["aaaatest"]],
    "__mul__": [[2], [3]],
    "__ne__": [["george"], ["test"]],
    "__radd__": [["str_1"], [""]],
    "__rmul__": [[2], [3]],
    "capitalize": [[]],
    "casefold": [[]],
    "center": [[15], [10, "0"]],
    "count": [["a"], ["g"], ["g", 5, 10]],
    "endswith": [["or"], ["ge"]],
    "expandtabs": [[]],
    "find": [["test"], ["tudor"], ["george"]],
    "index": [["test"]],
    "isalnum": [[]],
    "isalpha": [[]],
    "isascii": [[]],
    "isdecimal": [[]],
    "isdigit": [[]],
    "isidentifier": [[]],
    "islower": [[]],
    "isnumeric": [[]],
    "isprintable": [[]],
    "isspace": [[]],
    "istitle": [[]],
    "isupper": [[]],
    "join": [[["test", "me"]], [("tst", "m")]],
    "ljust": [[10], [25]],
    "lower": [[]],
    "lstrip": [[]],
    "partition": [["a"], ["t"], ["\t"]],
    "replace": [["tud", "dut"], ["AN", "KEY"]],
    "rfind": [["test"], ["tudor"], ["george"]],
    "rindex": [["test"]],
    "rjust": [[10], [25]],
    "rpartition": [["a"], ["t"], ["\t"]],
    "rsplit": [[], ["a"]],
    "rstrip": [["e"], ["r"], []],
    "split": [[], ["a"]],
    "splitlines": [[]],
    "startswith": [["j"], ["O"]],
    "strip": [["t"], ["r"], []],
    "swapcase": [[]],
    "title": [[]],
    "upper": [[]],
    "zfill": [[5], [10]],
}

objects = ["Op en Min ed", "george", "J A Y", "ANDREW", "tud\t\nor"]


@pytest.mark.parametrize("test_object", objects)
@pytest.mark.parametrize("func", inputs.keys())
def test_pointer_objectives(
    test_object, func, node: sy.VirtualMachine, client: sy.VirtualMachineClient
):
    remote_python = client.syft.lib.python.String
    local_constructor = sy.lib.python.String

    py_obj, sy_obj, remote_sy_obj = (
        test_object,
        local_constructor(test_object),
        remote_python(test_object),
    )
    possible_inputs = inputs[func]

    if not hasattr(py_obj, func):
        return

    py_method = getattr(py_obj, func)
    sy_method = getattr(sy_obj, func)

    if func == "__len__":
        func = "len"

    remote_sy_method = getattr(remote_sy_obj, func)

    for possible_input in possible_inputs:
        try:
            py_res = py_method(*possible_input)
        except Exception as py_e:
            py_res = str(py_e)

        try:
            sy_res = sy_method(*possible_input)
        except Exception as sy_e:
            sy_res = str(sy_e)

        try:
            remote_sy_res = remote_sy_method(*possible_input)
            get_permission(remote_sy_res, node)
            remote_sy_res = remote_sy_res.get()
        except Exception as remote_sy_e:
            remote_sy_res = str(remote_sy_e)

        assert py_res == sy_res
        assert sy_res == remote_sy_res



###############################################################################
# Language Modeling on Wikitext-2
#
# This file generates new sentences sampled from the language model
#
###############################################################################

# stdlib
import argparse

# third party
import torch

import data  # isort:skip

parser = argparse.ArgumentParser(description="PyTorch Wikitext-2 Language Model")

# Model parameters.
parser.add_argument(
    "--data",
    type=str,
    default="./data/wikitext-2",
    help="location of the data corpus; default: \"./data/wikitext-2\""
)
parser.add_argument(
    "--checkpoint",
    type=str,
    default="./model.pt",
    help="model checkpoint to use; default: \"./model.pt\""
)
parser.add_argument(
    "--outf",
    type=str,
    default="generated.txt",
    help="output file for generated text; default: \"generated.txt\""
)
parser.add_argument(
    "--words",
    type=int,
    default="1000",
    help="number of words to generate; default: 1000"
)
parser.add_argument(
    "--seed",
    type=int,
    default=1111,
    help="random seed; default: 1111"
)
parser.add_argument(
    "--cuda",
    action="store_true",
    help="use CUDA"
)
parser.add_argument(
    "--temperature",
    type=float,
    default=1.0,
    help="temperature - higher will increase diversity; default: 1.0"
)
parser.add_argument(
    "--log-interval",
    type=int,
    default=100,
    help="reporting interval; default: 100")
args = parser.parse_args()

# Set the random seed manually for reproducibility.
torch.manual_seed(args.seed)
if torch.cuda.is_available():
    if not args.cuda:
        print("WARNING: You have a CUDA device, so you should probably run with --cuda")

device = torch.device("cuda" if args.cuda else "cpu")

if args.temperature < 1e-3:
    parser.error("--temperature has to be greater or equal 1e-3")

with open(args.checkpoint, "rb") as f:
    model = torch.load(f).to(device)
model.eval()

corpus = data.Corpus(args.data)
ntokens = len(corpus.dictionary)

is_transformer_model = (
    hasattr(model, "model_type") and model.model_type == "Transformer"
)
if not is_transformer_model:
    hidden = model.init_hidden(1)
input = torch.randint(ntokens, (1, 1), dtype=torch.long).to(device)

with open(args.outf, "w") as outf:
    with torch.no_grad():  # no tracking history
        for i in range(args.words):
            if is_transformer_model:
                output = model(input, False)
                word_weights = output[-1].squeeze().div(args.temperature).exp().cpu()
                word_idx = torch.multinomial(word_weights, 1)[0]
                word_tensor = torch.Tensor([[word_idx]]).long().to(device)
                input = torch.cat([input, word_tensor], 0)
            else:
                output, hidden = model(input, hidden)
                word_weights = output.squeeze().div(args.temperature).exp().cpu()
                word_idx = torch.multinomial(word_weights, 1)[0]
                input.fill_(word_idx)

            word = corpus.dictionary.idx2word[word_idx]

            outf.write(word + ("\n" if i % 20 == 19 else " "))

            if i % args.log_interval == 0:
                print(f"| Generated {i}/{args.words} words")



# syft absolute
from syft.lib.python.tuple import Tuple

type2test = Tuple


def test_constructors():
    assert Tuple() == ()
    assert Tuple([]) == ()
    assert Tuple([0, 1, 2, 3]) == (0, 1, 2, 3)
    assert Tuple("") == ()
    assert Tuple("spam") == ("s", "p", "a", "m")
    assert Tuple(x for x in range(10) if x % 2) == (1, 3, 5, 7, 9)


def test_truth():
    assert not Tuple()
    assert Tuple((42,))


def test_len():
    assert len(()) == 0
    assert len((0,)) == 1
    assert len((0, 1, 2)) == 3


def test_iadd():
    u = (0, 1)
    u2 = u
    u += (2, 3)
    assert u is not u2


def test_imul():
    u = (0, 1)
    u2 = u
    u *= 3
    assert u is not u2


def test_repr():
    l0 = Tuple()
    l2 = (0, 1, 2)
    a0 = Tuple(l0)
    a2 = Tuple(l2)

    assert str(a0) == repr(l0)
    assert str(a2) == repr(l2)
    assert repr(a0) == "()"
    assert repr(a2), "(0, 1 ==  2)"


def test_hash():
    assert hash(Tuple()) == hash(())
    assert hash(Tuple((1,))) == hash((1,))
    assert hash(Tuple((-1, 0, 1))) == hash((-1, 0, 1))



# stdlib
from typing import Callable
from typing import List
from typing import Tuple

# syft relative
from .duet_init_test import test_scenario_init
from .duet_sanity_test import test_scenario_sanity
from .duet_torch_test import test_scenario_torch_tensor_sanity


def register_duet_scenarios(
    registered_tests: List[Tuple[str, Callable, Callable]]
) -> None:
    registered_tests.append(test_scenario_init)
    registered_tests.append(test_scenario_sanity)
    registered_tests.append(test_scenario_torch_tensor_sanity)



def do_test(port: int) -> None:
    # syft absolute
    import syft as sy

    duet = sy.launch_duet(loopback=True, network_url=f"http://127.0.0.1:{port}/")
    _ = sy.lib.python.List([1, 2, 3]).send(duet)

    sy.core.common.event_loop.loop.run_forever()


def ds_test(port: int) -> None:
    # syft absolute
    import syft as sy

    _ = sy.join_duet(loopback=True, network_url=f"http://127.0.0.1:{port}/")


test_scenario_init = ("test_scenario_init", do_test, ds_test)



# third party
import requests

# syft relative
from ...core.common.message import SignedEventualSyftMessageWithoutReply
from ...core.common.message import SignedImmediateSyftMessageWithReply
from ...core.common.message import SignedImmediateSyftMessageWithoutReply
from ...core.common.message import SyftMessage
from ...core.common.serde.deserialize import _deserialize
from ...core.common.serde.serialize import _serialize
from ...core.io.connection import ClientConnection
from ...proto.core.node.common.metadata_pb2 import Metadata as Metadata_PB


class HTTPConnection(ClientConnection):
    def __init__(self, url: str) -> None:
        self.base_url = url

    def send_immediate_msg_with_reply(
        self, msg: SignedImmediateSyftMessageWithReply
    ) -> SignedImmediateSyftMessageWithoutReply:
        """
        Sends high priority messages and wait for their responses.

        This method implements a HTTP version of the
        ClientConnection.send_immediate_msg_with_reply

        :return: returns an instance of SignedImmediateSyftMessageWithReply.
        :rtype: SignedImmediateSyftMessageWithoutReply
        """

        # Serializes SignedImmediateSyftMessageWithReply
        # and send it using HTTP protocol
        blob = self._send_msg(msg=msg).content
        # Deserialize node's response
        response = _deserialize(blob=blob, from_bytes=True)
        # Return SignedImmediateSyftMessageWithoutReply
        return response

    def send_immediate_msg_without_reply(
        self, msg: SignedImmediateSyftMessageWithoutReply
    ) -> None:
        """
        Sends high priority messages without waiting for their reply.

        This method implements a HTTP version of the
        ClientConnection.send_immediate_msg_without_reply

        """
        # Serializes SignedImmediateSyftMessageWithoutReply
        # and send it using HTTP protocol
        self._send_msg(msg=msg)

    def send_eventual_msg_without_reply(
        self, msg: SignedEventualSyftMessageWithoutReply
    ) -> None:
        """
        Sends low priority messages without waiting for their reply.

        This method implements a HTTP version of the
        ClientConnection.send_eventual_msg_without_reply
        """
        # Serializes SignedEventualSyftMessageWithoutReply in json format
        # and send it using HTTP protocol
        self._send_msg(msg=msg)

    def _send_msg(self, msg: SyftMessage) -> requests.Response:
        """
        Serializes Syft messages in json format and send it using HTTP protocol.

        NOTE: Auxiliary method to avoid code duplication and modularity.

        :return: returns requests.Response object containing a JSON serialized
        SyftMessage
        :rtype: requests.Response
        """

        # Perform HTTP request using base_url as a root address
        data_bytes: bytes = _serialize(msg, to_bytes=True)  # type: ignore
        r = requests.post(
            url=self.base_url,
            data=data_bytes,
            headers={"Content-Type": "application/octet-stream"},
        )

        # Return request's response object
        # r.text provides the response body as a str
        return r

    def _get_metadata(self) -> Metadata_PB:
        """
        Request Node's metadata

        :return: returns node metadata
        :rtype: str of bytes
        """
        data: bytes = requests.get(self.base_url + "/metadata").content
        metadata_pb = Metadata_PB()
        metadata_pb.ParseFromString(data)
        return metadata_pb



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/grid/messages/role_messages.proto
"""Generated protocol buffer code."""
# third party
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


# syft absolute
from syft.proto.core.common import (
    common_object_pb2 as proto_dot_core_dot_common_dot_common__object__pb2,
)
from syft.proto.core.io import address_pb2 as proto_dot_core_dot_io_dot_address__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name="proto/grid/messages/role_messages.proto",
    package="syft.grid.messages",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\'proto/grid/messages/role_messages.proto\x12\x12syft.grid.messages\x1a%proto/core/common/common_object.proto\x1a\x1bproto/core/io/address.proto"\x9c\x01\n\x11\x43reateRoleMessage\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12&\n\x07\x61\x64\x64ress\x18\x02 \x01(\x0b\x32\x15.syft.core.io.Address\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12\'\n\x08reply_to\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x89\x01\n\x12\x43reateRoleResponse\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12\x13\n\x0bstatus_code\x18\x02 \x01(\x05\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12&\n\x07\x61\x64\x64ress\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x99\x01\n\x0eGetRoleMessage\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12&\n\x07\x61\x64\x64ress\x18\x02 \x01(\x0b\x32\x15.syft.core.io.Address\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12\'\n\x08reply_to\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x86\x01\n\x0fGetRoleResponse\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12\x13\n\x0bstatus_code\x18\x02 \x01(\x05\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12&\n\x07\x61\x64\x64ress\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x9a\x01\n\x0fGetRolesMessage\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12&\n\x07\x61\x64\x64ress\x18\x02 \x01(\x0b\x32\x15.syft.core.io.Address\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12\'\n\x08reply_to\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x87\x01\n\x10GetRolesResponse\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12\x13\n\x0bstatus_code\x18\x02 \x01(\x05\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12&\n\x07\x61\x64\x64ress\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x9c\x01\n\x11SearchRoleMessage\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12&\n\x07\x61\x64\x64ress\x18\x02 \x01(\x0b\x32\x15.syft.core.io.Address\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12\'\n\x08reply_to\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x89\x01\n\x12SearchRoleResponse\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12\x13\n\x0bstatus_code\x18\x02 \x01(\x05\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12&\n\x07\x61\x64\x64ress\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x9c\x01\n\x11UpdateRoleMessage\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12&\n\x07\x61\x64\x64ress\x18\x02 \x01(\x0b\x32\x15.syft.core.io.Address\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12\'\n\x08reply_to\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x89\x01\n\x12UpdateRoleResponse\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12\x13\n\x0bstatus_code\x18\x02 \x01(\x05\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12&\n\x07\x61\x64\x64ress\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x9c\x01\n\x11\x44\x65leteRoleMessage\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12&\n\x07\x61\x64\x64ress\x18\x02 \x01(\x0b\x32\x15.syft.core.io.Address\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12\'\n\x08reply_to\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x89\x01\n\x12\x44\x65leteRoleResponse\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12\x13\n\x0bstatus_code\x18\x02 \x01(\x05\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12&\n\x07\x61\x64\x64ress\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Addressb\x06proto3',
    dependencies=[
        proto_dot_core_dot_common_dot_common__object__pb2.DESCRIPTOR,
        proto_dot_core_dot_io_dot_address__pb2.DESCRIPTOR,
    ],
)


_CREATEROLEMESSAGE = _descriptor.Descriptor(
    name="CreateRoleMessage",
    full_name="syft.grid.messages.CreateRoleMessage",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.CreateRoleMessage.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.CreateRoleMessage.address",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.CreateRoleMessage.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="reply_to",
            full_name="syft.grid.messages.CreateRoleMessage.reply_to",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=132,
    serialized_end=288,
)


_CREATEROLERESPONSE = _descriptor.Descriptor(
    name="CreateRoleResponse",
    full_name="syft.grid.messages.CreateRoleResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.CreateRoleResponse.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="status_code",
            full_name="syft.grid.messages.CreateRoleResponse.status_code",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.CreateRoleResponse.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.CreateRoleResponse.address",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=291,
    serialized_end=428,
)


_GETROLEMESSAGE = _descriptor.Descriptor(
    name="GetRoleMessage",
    full_name="syft.grid.messages.GetRoleMessage",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.GetRoleMessage.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.GetRoleMessage.address",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.GetRoleMessage.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="reply_to",
            full_name="syft.grid.messages.GetRoleMessage.reply_to",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=431,
    serialized_end=584,
)


_GETROLERESPONSE = _descriptor.Descriptor(
    name="GetRoleResponse",
    full_name="syft.grid.messages.GetRoleResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.GetRoleResponse.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="status_code",
            full_name="syft.grid.messages.GetRoleResponse.status_code",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.GetRoleResponse.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.GetRoleResponse.address",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=587,
    serialized_end=721,
)


_GETROLESMESSAGE = _descriptor.Descriptor(
    name="GetRolesMessage",
    full_name="syft.grid.messages.GetRolesMessage",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.GetRolesMessage.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.GetRolesMessage.address",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.GetRolesMessage.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="reply_to",
            full_name="syft.grid.messages.GetRolesMessage.reply_to",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=724,
    serialized_end=878,
)


_GETROLESRESPONSE = _descriptor.Descriptor(
    name="GetRolesResponse",
    full_name="syft.grid.messages.GetRolesResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.GetRolesResponse.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="status_code",
            full_name="syft.grid.messages.GetRolesResponse.status_code",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.GetRolesResponse.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.GetRolesResponse.address",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=881,
    serialized_end=1016,
)


_SEARCHROLEMESSAGE = _descriptor.Descriptor(
    name="SearchRoleMessage",
    full_name="syft.grid.messages.SearchRoleMessage",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.SearchRoleMessage.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.SearchRoleMessage.address",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.SearchRoleMessage.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="reply_to",
            full_name="syft.grid.messages.SearchRoleMessage.reply_to",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1019,
    serialized_end=1175,
)


_SEARCHROLERESPONSE = _descriptor.Descriptor(
    name="SearchRoleResponse",
    full_name="syft.grid.messages.SearchRoleResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.SearchRoleResponse.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="status_code",
            full_name="syft.grid.messages.SearchRoleResponse.status_code",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.SearchRoleResponse.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.SearchRoleResponse.address",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1178,
    serialized_end=1315,
)


_UPDATEROLEMESSAGE = _descriptor.Descriptor(
    name="UpdateRoleMessage",
    full_name="syft.grid.messages.UpdateRoleMessage",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.UpdateRoleMessage.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.UpdateRoleMessage.address",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.UpdateRoleMessage.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="reply_to",
            full_name="syft.grid.messages.UpdateRoleMessage.reply_to",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1318,
    serialized_end=1474,
)


_UPDATEROLERESPONSE = _descriptor.Descriptor(
    name="UpdateRoleResponse",
    full_name="syft.grid.messages.UpdateRoleResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.UpdateRoleResponse.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="status_code",
            full_name="syft.grid.messages.UpdateRoleResponse.status_code",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.UpdateRoleResponse.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.UpdateRoleResponse.address",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1477,
    serialized_end=1614,
)


_DELETEROLEMESSAGE = _descriptor.Descriptor(
    name="DeleteRoleMessage",
    full_name="syft.grid.messages.DeleteRoleMessage",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.DeleteRoleMessage.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.DeleteRoleMessage.address",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.DeleteRoleMessage.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="reply_to",
            full_name="syft.grid.messages.DeleteRoleMessage.reply_to",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1617,
    serialized_end=1773,
)


_DELETEROLERESPONSE = _descriptor.Descriptor(
    name="DeleteRoleResponse",
    full_name="syft.grid.messages.DeleteRoleResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.DeleteRoleResponse.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="status_code",
            full_name="syft.grid.messages.DeleteRoleResponse.status_code",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.DeleteRoleResponse.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.DeleteRoleResponse.address",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1776,
    serialized_end=1913,
)

_CREATEROLEMESSAGE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_CREATEROLEMESSAGE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_CREATEROLEMESSAGE.fields_by_name[
    "reply_to"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_CREATEROLERESPONSE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_CREATEROLERESPONSE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_GETROLEMESSAGE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_GETROLEMESSAGE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_GETROLEMESSAGE.fields_by_name[
    "reply_to"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_GETROLERESPONSE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_GETROLERESPONSE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_GETROLESMESSAGE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_GETROLESMESSAGE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_GETROLESMESSAGE.fields_by_name[
    "reply_to"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_GETROLESRESPONSE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_GETROLESRESPONSE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_SEARCHROLEMESSAGE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_SEARCHROLEMESSAGE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_SEARCHROLEMESSAGE.fields_by_name[
    "reply_to"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_SEARCHROLERESPONSE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_SEARCHROLERESPONSE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_UPDATEROLEMESSAGE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_UPDATEROLEMESSAGE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_UPDATEROLEMESSAGE.fields_by_name[
    "reply_to"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_UPDATEROLERESPONSE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_UPDATEROLERESPONSE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_DELETEROLEMESSAGE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_DELETEROLEMESSAGE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_DELETEROLEMESSAGE.fields_by_name[
    "reply_to"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_DELETEROLERESPONSE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_DELETEROLERESPONSE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
DESCRIPTOR.message_types_by_name["CreateRoleMessage"] = _CREATEROLEMESSAGE
DESCRIPTOR.message_types_by_name["CreateRoleResponse"] = _CREATEROLERESPONSE
DESCRIPTOR.message_types_by_name["GetRoleMessage"] = _GETROLEMESSAGE
DESCRIPTOR.message_types_by_name["GetRoleResponse"] = _GETROLERESPONSE
DESCRIPTOR.message_types_by_name["GetRolesMessage"] = _GETROLESMESSAGE
DESCRIPTOR.message_types_by_name["GetRolesResponse"] = _GETROLESRESPONSE
DESCRIPTOR.message_types_by_name["SearchRoleMessage"] = _SEARCHROLEMESSAGE
DESCRIPTOR.message_types_by_name["SearchRoleResponse"] = _SEARCHROLERESPONSE
DESCRIPTOR.message_types_by_name["UpdateRoleMessage"] = _UPDATEROLEMESSAGE
DESCRIPTOR.message_types_by_name["UpdateRoleResponse"] = _UPDATEROLERESPONSE
DESCRIPTOR.message_types_by_name["DeleteRoleMessage"] = _DELETEROLEMESSAGE
DESCRIPTOR.message_types_by_name["DeleteRoleResponse"] = _DELETEROLERESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CreateRoleMessage = _reflection.GeneratedProtocolMessageType(
    "CreateRoleMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _CREATEROLEMESSAGE,
        "__module__": "proto.grid.messages.role_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.CreateRoleMessage)
    },
)
_sym_db.RegisterMessage(CreateRoleMessage)

CreateRoleResponse = _reflection.GeneratedProtocolMessageType(
    "CreateRoleResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _CREATEROLERESPONSE,
        "__module__": "proto.grid.messages.role_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.CreateRoleResponse)
    },
)
_sym_db.RegisterMessage(CreateRoleResponse)

GetRoleMessage = _reflection.GeneratedProtocolMessageType(
    "GetRoleMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETROLEMESSAGE,
        "__module__": "proto.grid.messages.role_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.GetRoleMessage)
    },
)
_sym_db.RegisterMessage(GetRoleMessage)

GetRoleResponse = _reflection.GeneratedProtocolMessageType(
    "GetRoleResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETROLERESPONSE,
        "__module__": "proto.grid.messages.role_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.GetRoleResponse)
    },
)
_sym_db.RegisterMessage(GetRoleResponse)

GetRolesMessage = _reflection.GeneratedProtocolMessageType(
    "GetRolesMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETROLESMESSAGE,
        "__module__": "proto.grid.messages.role_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.GetRolesMessage)
    },
)
_sym_db.RegisterMessage(GetRolesMessage)

GetRolesResponse = _reflection.GeneratedProtocolMessageType(
    "GetRolesResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETROLESRESPONSE,
        "__module__": "proto.grid.messages.role_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.GetRolesResponse)
    },
)
_sym_db.RegisterMessage(GetRolesResponse)

SearchRoleMessage = _reflection.GeneratedProtocolMessageType(
    "SearchRoleMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _SEARCHROLEMESSAGE,
        "__module__": "proto.grid.messages.role_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.SearchRoleMessage)
    },
)
_sym_db.RegisterMessage(SearchRoleMessage)

SearchRoleResponse = _reflection.GeneratedProtocolMessageType(
    "SearchRoleResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _SEARCHROLERESPONSE,
        "__module__": "proto.grid.messages.role_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.SearchRoleResponse)
    },
)
_sym_db.RegisterMessage(SearchRoleResponse)

UpdateRoleMessage = _reflection.GeneratedProtocolMessageType(
    "UpdateRoleMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _UPDATEROLEMESSAGE,
        "__module__": "proto.grid.messages.role_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.UpdateRoleMessage)
    },
)
_sym_db.RegisterMessage(UpdateRoleMessage)

UpdateRoleResponse = _reflection.GeneratedProtocolMessageType(
    "UpdateRoleResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _UPDATEROLERESPONSE,
        "__module__": "proto.grid.messages.role_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.UpdateRoleResponse)
    },
)
_sym_db.RegisterMessage(UpdateRoleResponse)

DeleteRoleMessage = _reflection.GeneratedProtocolMessageType(
    "DeleteRoleMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _DELETEROLEMESSAGE,
        "__module__": "proto.grid.messages.role_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.DeleteRoleMessage)
    },
)
_sym_db.RegisterMessage(DeleteRoleMessage)

DeleteRoleResponse = _reflection.GeneratedProtocolMessageType(
    "DeleteRoleResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _DELETEROLERESPONSE,
        "__module__": "proto.grid.messages.role_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.DeleteRoleResponse)
    },
)
_sym_db.RegisterMessage(DeleteRoleResponse)


# @@protoc_insertion_point(module_scope)



# third party
import pytest
import torch as th

# syft absolute
import syft as sy


@pytest.fixture(scope="function")
def tensor() -> th.Tensor:
    t1 = th.tensor([[1.0, -1.0], [1.0, -1.0]])
    scale, zero_point = 1e-4, 2
    dtype = th.qint32
    tensor = th.quantize_per_tensor(t1, scale, zero_point, dtype)
    return tensor


def test_protobuf_tensor_serializer_deserializer(tensor: th.Tensor) -> None:
    tensor2 = sy.lib.torch.tensor_util.protobuf_tensor_serializer(tensor)
    assert tensor2.is_quantized is True
    assert tuple(tensor2.shape) == tuple(tensor.shape)
    tensor2_serial = sy.lib.torch.tensor_util.protobuf_tensor_deserializer(tensor2)
    assert tensor2_serial.is_quantized is True
    assert tuple(tensor2_serial.shape) == tuple(tensor.shape)



# stdlib
import logging

# third party
import _pytest

# syft absolute
import syft as sy


def test_load_lib() -> None:
    assert sy.lib.load(lib="tenseal") is None
    assert sy.lib.load("tenseal", "opacus") is None
    assert sy.lib.load(["tenseal", "opacus"]) is None
    assert sy.lib.load(("tenseal", "opacus")) is None
    assert sy.lib.load({"tenseal"}) is None


def test_load_errors(caplog: _pytest.logging.LogCaptureFixture) -> None:
    # Error if a non-supported library is loaded
    with caplog.at_level(logging.CRITICAL, logger="syft.logger"):
        sy.lib.load("non_compatible")
    assert "Unable to load package support for: non_compatible." in caplog.text
    caplog.clear()

    # Error if non-string object type is attempted to be loaded
    with caplog.at_level(logging.CRITICAL, logger="syft.logger"):
        sy.lib.load([True])
    assert "Unable to load package support for: True." in caplog.text
    caplog.clear()

    # Error if a non-iterable object is passed
    with caplog.at_level(logging.CRITICAL, logger="syft.logger"):
        sy.lib.load(True)
    assert "Unable to load package support for any library." in caplog.text
    caplog.clear()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/core/node/common/service/object_search_message.proto
"""Generated protocol buffer code."""
# third party
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


# syft absolute
from syft.proto.core.common import (
    common_object_pb2 as proto_dot_core_dot_common_dot_common__object__pb2,
)
from syft.proto.core.io import address_pb2 as proto_dot_core_dot_io_dot_address__pb2
from syft.proto.core.pointer import (
    pointer_pb2 as proto_dot_core_dot_pointer_dot_pointer__pb2,
)

DESCRIPTOR = _descriptor.FileDescriptor(
    name="proto/core/node/common/service/object_search_message.proto",
    package="syft.core.node.common.service",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n:proto/core/node/common/service/object_search_message.proto\x12\x1dsyft.core.node.common.service\x1a%proto/core/common/common_object.proto\x1a\x1bproto/core/io/address.proto\x1a proto/core/pointer/pointer.proto"\x8d\x01\n\x13ObjectSearchMessage\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12&\n\x07\x61\x64\x64ress\x18\x02 \x01(\x0b\x32\x15.syft.core.io.Address\x12\'\n\x08reply_to\x18\x03 \x01(\x0b\x32\x15.syft.core.io.Address"\x96\x01\n\x18ObjectSearchReplyMessage\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12&\n\x07\x61\x64\x64ress\x18\x02 \x01(\x0b\x32\x15.syft.core.io.Address\x12+\n\x07results\x18\x03 \x03(\x0b\x32\x1a.syft.core.pointer.Pointerb\x06proto3',
    dependencies=[
        proto_dot_core_dot_common_dot_common__object__pb2.DESCRIPTOR,
        proto_dot_core_dot_io_dot_address__pb2.DESCRIPTOR,
        proto_dot_core_dot_pointer_dot_pointer__pb2.DESCRIPTOR,
    ],
)


_OBJECTSEARCHMESSAGE = _descriptor.Descriptor(
    name="ObjectSearchMessage",
    full_name="syft.core.node.common.service.ObjectSearchMessage",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.core.node.common.service.ObjectSearchMessage.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.core.node.common.service.ObjectSearchMessage.address",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="reply_to",
            full_name="syft.core.node.common.service.ObjectSearchMessage.reply_to",
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=196,
    serialized_end=337,
)


_OBJECTSEARCHREPLYMESSAGE = _descriptor.Descriptor(
    name="ObjectSearchReplyMessage",
    full_name="syft.core.node.common.service.ObjectSearchReplyMessage",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.core.node.common.service.ObjectSearchReplyMessage.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.core.node.common.service.ObjectSearchReplyMessage.address",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="results",
            full_name="syft.core.node.common.service.ObjectSearchReplyMessage.results",
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=340,
    serialized_end=490,
)

_OBJECTSEARCHMESSAGE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_OBJECTSEARCHMESSAGE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_OBJECTSEARCHMESSAGE.fields_by_name[
    "reply_to"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_OBJECTSEARCHREPLYMESSAGE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_OBJECTSEARCHREPLYMESSAGE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_OBJECTSEARCHREPLYMESSAGE.fields_by_name[
    "results"
].message_type = proto_dot_core_dot_pointer_dot_pointer__pb2._POINTER
DESCRIPTOR.message_types_by_name["ObjectSearchMessage"] = _OBJECTSEARCHMESSAGE
DESCRIPTOR.message_types_by_name["ObjectSearchReplyMessage"] = _OBJECTSEARCHREPLYMESSAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ObjectSearchMessage = _reflection.GeneratedProtocolMessageType(
    "ObjectSearchMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _OBJECTSEARCHMESSAGE,
        "__module__": "proto.core.node.common.service.object_search_message_pb2"
        # @@protoc_insertion_point(class_scope:syft.core.node.common.service.ObjectSearchMessage)
    },
)
_sym_db.RegisterMessage(ObjectSearchMessage)

ObjectSearchReplyMessage = _reflection.GeneratedProtocolMessageType(
    "ObjectSearchReplyMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _OBJECTSEARCHREPLYMESSAGE,
        "__module__": "proto.core.node.common.service.object_search_message_pb2"
        # @@protoc_insertion_point(class_scope:syft.core.node.common.service.ObjectSearchReplyMessage)
    },
)
_sym_db.RegisterMessage(ObjectSearchReplyMessage)


# @@protoc_insertion_point(module_scope)
